function ControlMuscleGuessOgden2net
%% Two NETWORK OGDEN MODEL
% Anil Palaparthi 20100208
% after JASA paper Eq. (6) (Zhang et al., 2006)

%% General Constants
tau_ = 1/250;
freq_ = 1;
eps_amp_ = 28;  % percent

%% Fit Workcycle to Ogden Passive
rng = [1];
guess0_(1,:)   =[0.01].*rng;     % Z - the viscosity scaling factor of network (B), see JASA paper Eq. (9) (Zhang et al., 2006)
guess0_(2,:)   =[-1].*rng;     % c - the stretch exponent of network (B), set to -1, see JASA paper Eq. (9) (Zhang et al., 2006)
guess0_(3,:)   =[2.1].*rng;     % m_B - the stress exponent of network (B), see JASA paper Eq. (9) (Zhang et al., 2006)
guess0_(4,:)   =[6.39].*rng;    %  mu_A -the initial shear modulus of equilibrium network (A), see JASA paper Eq. (6) (Zhang et al., 2006)
guess0_(5,:)   =[40.0].*rng;     % mu_B -the initial shear modulus of short-term network (B), see JASA paper Eq. (12) (Zhang et al., 2006)
guess0_(6,:)   =[14.4].*rng;     % a - alpha; dimensionless power describing the nonlinearity, see JASA paper Eq. (6) (Zhang et al., 2006)

%% Different Inputs
time_ = 0:tau_:2.5;
eps_= eps_amp_/100*(1-(.5+.5*cos(2*pi*time_*freq_)));
%eps_ = 0.2*ones(1,length(time_));
%eps_ = (time_<=8).*0.04.*sin(2*pi*6.5*time_)+0.2*ones(1,length(time_));

%% Calculate needed stuff
lamda_A=1+eps_+0.0001;    % lamda = 1+eps
dlamda_A=0.*lamda_A;
dlamda_A(2:end)=diff(lamda_A)/tau_;	% first point is ignored
dlamda_A(1)=dlamda_A(2);
% figure(3)  % strain vs stretch
% plot(time_,eps_,time_,lamda_A)
% title('strain vs stretch')

%Function to obtain stress data
sigT(1,:) = model_bananafunc([guess0_(:,1)'],tau_,lamda_A,dlamda_A);

% Store the stress and strain data in a file
y = [eps_;sigT]; 
fid = fopen('anilogden.txt','wt');
fprintf(fid,'%12.8f %12.8f\n',y);
fclose(fid);

%plot the stress vs strain
figure(1),plot(eps_,sigT,'b-')
title('stress vs strain')
xlabel('strain (\epsilon)')
ylabel('stress (\sigma) in kPa')

figure(2), plot(time_,sigT,'b-')
xlabel('time (sec)');
ylabel('stress (\sigma) in kPa');
keyboard


%% solve for lamda_B and sigT
function sigT = model_bananafunc(par,tau,lamda_A,dlamda_A)
% Function to solve for lamda_B and obtain total stress data
% for a two network ogden model
%
% par - seed parameters
% tau - sampling rate
% lamda_A - strech of network A
% dlamda_A - derivative of strech of network A
%
% returns total stress in network A and network B
%%%%%%%%%%%%%%%
%
%   Anil Palaparthi 20100208
%
lamda_B=1;  % Initial value for lamda_B

%Store the parameters in temporary variables
Z = par(1,1);c = par(1,2); m_B = par(1,3); mu_A = par(1,4); mu_B = par(1,5);
a = par(1,6);

% Initial value for derivative of lamda_B
dlamda_B = [dlamda_A(1)*(lamda_B/lamda_A(1))]-...
           (2/3)*lamda_B*Z*sign(lamda_B-1+1e-15)*...
           [[sqrt((1/3)*[(lamda_A(1)/lamda_B)^2+(2*lamda_B/lamda_A(1))])-1+1e-4]^c]*...
           [abs((2*mu_B/a)*[(lamda_B^a)-(lamda_B^(-0.5*a))])]^m_B;
 
ns = length(lamda_A);
lambda_B=zeros(1,ns); %array to store lamda_B data 
lambda_B(1) = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% initializations for RK4for.m DO NOT CHANGE ANYTHING %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phi = lamda_B;  % RK routine internal variable
yo = phi;       % RK routine internal variable
N = 1;          % number of equations solved by RK4
tmp = 1;        % temporary counting array
c1 = tmp+1;     % indecies for regaining above variables
c3 = tmp+1+2*N; c4 = tmp+1+3*N;  %_from RK4 output 
time = 0;       % time set to zero,
sigT = zeros(1,ns);  %Initialize total stress
sigT(1) = (2*mu_A/a)*[(lamda_A(1))^(a-1)-(lamda_A(1))^(-0.5*a-1)]+...
    (2*mu_B/a)*[(lamda_B)^(a-1)-(lamda_B)^(-0.5*a-1)];

for n=2:ns
    
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %% Runge-Kutta 4 routine...
   %%
   
   Istat = 1;
   m = 0;
   while(Istat~=0),
       m = m+1;
       [temp] = RKfor(N,lamda_B, dlamda_B, time, tau, phi, yo, m);
       %% send [(N,F,dF,time,tau,phi,yo,m)
       % ...N = number of variables to solve, 
       % ...F = variable to solve
       % ...dF= its derivative
       % ...time= current time in iteration
       % ...tau = time step,
       % ...internal parameter for recall
       % ...internal parameter for recall].
       % ...Returns updated... [status F dF phi yo time]
       %% regain variables 
       Istat = temp(1);     %% condition of loop
       lamda_B = temp(c1);  %% update the RKA variables
       phi = temp(c3);      %% internal variable for recall
       yo = temp(c4);       %% internal variable for recall
       time = temp(end);    %% time update
       
       % derivative of lamda_B: equation 11 from (Zhang et al., 2006)
       dlamda_B(1) = [dlamda_A(n)*(lamda_B(1)/lamda_A(n-1))]-...
           [(2/3)*lamda_B(1)*Z*sign(lamda_B(1)-1+1e-15)*...
           [[sqrt((1/3)*[(lamda_A(n-1)/lamda_B(1))^2+(2*lamda_B(1)/lamda_A(n-1))])-1+1e-4]^c]*...
           [abs((2*mu_B/a)*[(lamda_B(1)^a)-(lamda_B(1)^(-0.5*a))])]^m_B];
 
   end
   %%
   %%  end runge-kutta 
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
   lambda_B(n) = lamda_B;  % Store the value of lamda_B
   
   % Total Stress for network A and network B
   sigT(n) = (2*mu_A/a)*[(lamda_A(n))^(a-1)-(lamda_A(n))^(-0.5*a-1)]+...
       (2*mu_B/a)*[(lamda_B)^(a-1)-(lamda_B)^(-0.5*a-1)]; 
end
return



function out = RKfor( N,Y,F,X,H,PHI,Y0,m) 
%RKfor
%
%	Runge-Kutte 4th order routine based on a fortran 
%		version used by Ingo Titze.  
%
%	Eric Hunter did the conversion
%
J=1:N;
switch m
case 1,
   istat=1;
case 2,
   istat=1;
   Y0(J)=Y(J);
   PHI(J)=F(J);
   Y(J)=Y0(J)+0.5*H*F(J);
   X=X+0.5*H;
case 3,
   istat=1;
   PHI(J)=PHI(J)+2.*F(J);
   Y(J)=Y0(J)+0.5*H*F(J);
case 4,
   istat=1;
   PHI(J)=PHI(J)+2.*F(J);
   Y(J)=Y0(J)+H*F(J);
   X=X+0.5*H;
otherwise,
   istat=0;
   Y(J)=Y0(J)+(PHI(J)+F(J))*H/6;           
end
out=[istat,Y,F,PHI,Y0,X];
return
%----------------------------------------------------------
