function ControlMuscleGuessOgden3net
%%  Three Network Ogden model
%   Anil Palaparthi 2010-02-25
%
%   The model characterizes the transient responses of soft tissues
%   such as cyclic stress relaxation and creep behavior
%   
%   Inputs:
%   parameters of the Three network model
%
%   Outputs:
%   The stress-strain curve representing the transient responses of 
%   soft tissues
%
%%  General Constants
    close all;
    clear all;
    clc;
    tau_ = 1/1000;          % sampling rate
    freq_ = 1;              % frequency of input strain
    
%%  Fit Workcycle to Ogden Model
    rng = [1];
%   parameters of network A
    guess0_(1,:) = [17.6746].*rng;             % a
    guess0_(2,:) = [4.743].*rng;             % mu_A
%   parameters of network B
    guess0_(3,:) = [0.022].*rng;             % Z_B 
    guess0_(4,:) = [-0.80902].*rng;               % c_B 
    guess0_(5,:) = [1.2527].*rng;              % m_B 
    guess0_(6,:) = [78.3221].*rng;             % mu_B 
%   parameters of network C    
    guess0_(7,:) = [.005].*rng;            % Z_C
    guess0_(8,:) = [-0.005].*rng;               % c_C
    guess0_(9,:) = [1.1].*rng;              % m_C
    guess0_(10,:) = [0.74].*rng;            % mu_C
    
% Z_cv=0.0025; m_cv=1.8; c_cv=-1; alpha_cv=15.0; m_B_cv=130.0;
% Z_lg=0.004; m_lg=1.3; c_lg=-1; alpha_lg=20.0; m_B_lg=50.0;
% 
% lamda_0=1.0001; lamda1_0=1; lamda2_0=1;
% C_1=2; C_2=2; C_3=-2; C_4=0; f_cv=1; f_lg=0;

    
%%  Inputs
    time_ = 0:tau_:50;
    eps_amp_ = 20.77;
    eps_= eps_amp_/100*(1-(.5+.5*cos(2*pi*time_*freq_)));
    
%%  Calculate needed stuff
    lamda_A=1+eps_+0.0001;                  % lamda = 1+eps
    dlamda_A=0.*lamda_A;
    dlamda_A(2:end)=diff(lamda_A)/tau_;     % first point is ignored
    dlamda_A(1)=dlamda_A(2);
        
%   plot the stretch across time
    figure(10),plot(time_,lamda_A)
    xlabel('time');ylabel('Stretch')
    
%   Function to obtain stress data for the given stretch
    sigT(1,:) = model_Ogden3NetFunction([guess0_(:,1)'],tau_,lamda_A,dlamda_A);

%   Store the stress and strain data in a file
    y = [eps_;sigT]; 
    fid = fopen('OgdenNetwork3.txt','wt');
    fprintf(fid,'%12.8f %12.8f\n',y);
    fclose(fid);

%%  plots
%   stress vs strain
    figure(20),plot(eps_,sigT,'b-')
    title('stress vs strain')
    xlabel('strain (\epsilon)')
    ylabel('stress (\sigma)')
    
%   Total stress vs time
    figure(30), plot(time_,sigT,'b-')
    xlabel('time (sec)');
    ylabel('stress(kPa)');
    
keyboard


%%  Solve for lamda_B and lamda_C and obtain total stress sigT
function sigT = model_Ogden3NetFunction(par,tau,lamda_A,dlamda_A)
%   Function to solve for lamda_B and lamda_C and obtain total stress data
%   for a three network ogden model
%
%   par - seed parameters
%   tau - sampling rate
%   lamda_A - strech of network A
%   dlamda_A - derivative of strech of network A
%
%   returns total stress in network A, network B and network C
%%%%%%%%%%%%%%%
%
%   Anil Palaparthi 2010-02-25
%

%%  Store the parameters of Ogden model in temporary variables
%   parameters of network A
    a = par(1,1);
    mu_A = par(1,2);
%   Parameters of network B
    Z_B = par(1,3);
    c_B = par(1,4);
    m_B = par(1,5);
    mu_B = par(1,6);
%   parameters of network C
    Z_C = par(1,7);
    c_C = par(1,8);
    m_C = par(1,9);
    mu_C = par(1,10);
    
%%  Initializations
    ns = length(lamda_A);
%   Network B
    lamda_B = 1;                            % Stretch of Network B
%   derivative of stretch of Network B
    dlamda_B = [dlamda_A(1)*(lamda_B/lamda_A(1))]-...
           (2/3)*lamda_B*Z_B*sign(lamda_B-1+1e-15)*...
           [[sqrt((1/3)*[(lamda_A(1)/lamda_B)^2+(2*lamda_B/lamda_A(1))])-1+1e-4]^c_B]*...
           [abs((2*mu_B/a)*[(lamda_B^a)-(lamda_B^(-0.5*a))])]^m_B;
                                                
%   Network C
    lamda_C = 1.1;
%   derivative of stretch of Network C
    dlamda_C = [dlamda_A(1)*(lamda_C/lamda_A(1))]-...
           (2/3)*lamda_C*Z_C*sign(lamda_C-1+1e-15)*...
           [[sqrt((1/3)*[(lamda_A(1)/lamda_C)^2+(2*lamda_C/lamda_A(1))])-1+1e-4]^c_C]*...
           [abs((2*mu_C/a)*[(lamda_C^a)-(lamda_C^(-0.5*a))])]^m_C;
       
%% 
%
%   Initializations for RK4for.m DO NOT CHANGE ANYTHING %%
%
%   Network B
    phi_B  = lamda_B;                       % RK routine internal variable
    yo_B   = phi_B;                         % RK routine internal variable
    time_B = 0;                             % time set to Zero  
%   Network C
    phi_C  = lamda_C;                       % RK routine internal variable
    yo_C   = phi_C;                         % RK routine internal varialbe
    time_C = 0;                             % time set to zero  
    
    N = 1;                                  % number of equs solved by RK4
    tmp = 1;                                % temporary counting array
    c1 = tmp+1;                             % indecies for regaining above
    c3 = tmp+1+2*N; c4 = tmp+1+3*N;         % _variables from RK4 output 

%   Total stress
    sigT = zeros(1,ns);                     %Initialize total stress
    sigT(1) = (2*mu_A/a)*[lamda_A(1)^(a-1) - lamda_A(1)^(-0.5*a-1)]+...
        (2*mu_B/a)*[lamda_B^(a-1) - lamda_B^(-0.5*a-1)]+...
        (2*mu_C/a)*[lamda_C^(a-1) - lamda_C^(-0.5*a-1)];
    
    for n = 2:ns
        
        Istat = 1;
        m = 0;
        
%       Network B
        while(Istat~=0),
            m = m+1;
            [temp]=RKfor(N,lamda_B, dlamda_B, time_B, tau, phi_B, yo_B, m);
            %% send [(N,F,dF,time,tau,phi,yo,m)
            % ...N = number of variables to solve, 
            % ...F = variable to solve
            % ...dF= its derivative
            % ...time= current time in iteration
            % ...tau = time step,
            % ...internal parameter for recall
            % ...internal parameter for recall].
            % ...Returns updated... [status F dF phi yo time]
            %% regain variables 
            
            Istat   = temp(1);              % condition of loop
            lamda_B = temp(c1);             % update the RKA variables
            phi_B   = temp(c3);             % internal variable for recall
            yo_B    = temp(c4);             % internal variable for recall
            time_B  = temp(end);            % time update
       
            % derivative of lamda_B: equation 11 from (Zhang et al., 2006)
            dlamda_B(1) = [dlamda_A(n)*(lamda_B(1)/lamda_A(n-1))]-...
                [(2/3)*lamda_B(1)*Z_B*sign(lamda_B(1)-1+1e-15)*...
                [[sqrt((1/3)*[(lamda_A(n-1)/lamda_B(1))^2+(2*lamda_B(1)/lamda_A(n-1))])-1+1e-4]^c_B]*...
                [abs((2*mu_B/a)*[(lamda_B(1)^a)-(lamda_B(1)^(-0.5*a))])]^m_B];
 
        end
        
        Istat = 1;
        m = 0;
        
%       Network C
        while(Istat~=0),
            m = m+1;
            [temp]=RKfor(N,lamda_C, dlamda_C, time_C, tau, phi_C, yo_C, m);
            %% send [(N,F,dF,time,tau,phi,yo,m)
            % ...N = number of variables to solve, 
            % ...F = variable to solve
            % ...dF= its derivative
            % ...time= current time in iteration
            % ...tau = time step,
            % ...internal parameter for recall
            % ...internal parameter for recall].
            % ...Returns updated... [status F dF phi yo time]
            %% regain variables 
            
            Istat   = temp(1);              % condition of loop
            lamda_C = temp(c1);                % update the RKA variables
            phi_C   = temp(c3);             % internal variable for recall
            yo_C    = temp(c4);             % internal variable for recall
            time_C  = temp(end);            % time update
       
            % derivative of lamda_B: equation 11 from (Zhang et al., 2006)
            dlamda_C(1) = [dlamda_A(n)*(lamda_C(1)/lamda_A(n-1))]-...
                [(2/3)*lamda_C(1)*Z_C*sign(lamda_C(1)-1+1e-15)*...
                [[sqrt((1/3)*[(lamda_A(n-1)/lamda_C(1))^2+(2*lamda_C(1)/lamda_A(n-1))])-1+1e-4]^c_C]*...
                [abs((2*mu_C/a)*[(lamda_C(1)^a)-(lamda_C(1)^(-0.5*a))])]^m_C];
 
        end
        
%   Total stress
    sigT(n) = (2*mu_A/a)*[lamda_A(n)^(a-1) - lamda_A(n)^(-0.5*a-1)]+...
        (2*mu_B/a)*[lamda_B^(a-1) - lamda_B^(-0.5*a-1)]+...
        (2*mu_C/a)*[lamda_C^(a-1) - lamda_C^(-0.5*a-1)];
    end
    
return


function out = RKfor( N,Y,F,X,H,PHI,Y0,m) 
%   RKfor
%
%	Runge-Kutte 4th order routine based on a fortran version
%		used by Ingo Titze.  From an old Fortran Numerical methods book
%
%	Eric Hunter did the conversion
%
    J=1:N;
    switch m
    case 1,
        istat=1;
    case 2,
        istat=1;
        Y0(J)=Y(J);
        PHI(J)=F(J);
        Y(J)=Y0(J)+0.5*H*F(J);
        X=X+0.5*H;
    case 3,
        istat=1;
        PHI(J)=PHI(J)+2.*F(J);
        Y(J)=Y0(J)+0.5*H*F(J);
    case 4,
        istat=1;
        PHI(J)=PHI(J)+2.*F(J);
        Y(J)=Y0(J)+H*F(J);
        X=X+0.5*H;
    otherwise,
        istat=0;
        Y(J)=Y0(J)+(PHI(J)+F(J))*H/6;           
    end
    out=[istat,Y,F,PHI,Y0,X];
return
%----------------------------------------------------------

