function ControlSimpleFitOgden
%	function that matches parameters of a tissue model to some muscle data
%	stress-strain curve
%
%	Eric Hunter; May 2002, May 2006
%   Anil Palaparthi: 20100208
%
%   uses a modified (by Eric Hunter) version of:
%    	Title:  	fminsearchbnd.
%   	Author: 	John D'Errico
%     Downloaded from Matlab.com Matlab Centeral
%     http://www.mathworks.com/matlabcentral/fileexchange
%	
%  Given a guess (seed parameters) muscle stress is predicted.  
%  
% RIGHT OF REFUSAL
%    The data, images and text are open to use by the public as a service 
% of the National Center for Voice and Speech.  However, we ask the reader 
% to respect the time and effort put into this manuscript and research.  
% If the text, images, scripts, or data are used, the user agrees to 
% reference to this document and the National Center for Voice and Speech. 
% We reserve the right of refusal to (1) be authors on papers using the 
% data and scripts and (2) have the supporting project acknowledged.  The 
% user agrees to freely share with the NCVS and the scientific community
% any data sets or extension software build on the data or scripts contained.
%
% ACKNOWLEDGEMENTS
%    This work was supported by NIDCD grant No DC04347 from National 
% Institutes of Health.  Thanks to the University of Utah VP for Research
% for their continued support as the NCVS UTAH site host.  The 
% authors would like to thank the research team at the National Center 
% for Voice and Speech with many supporting roles in this work.  Muscle data 
% was collected with support from DC004347 (NIH/NIDCD) and with special help
% form Fari Alipour, Doug Montequin, Roger Chan and Niro Tayama.  The 
% analysis and modeling presented here were conducted as part of a research
% program entitled, A Computational Tool for Simulation of Phonosurgical 
% Proceedures, also supported by the NIDCD (R03 DC006801). 


%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% General Constants
close all;
clear all;
clc;
num_itt_=10000;      % number of iterations to try and find fit

%%%%%%%%%%%%%%%%%%%%
%%%%%% fit workcycle to Ogden Model
%  the first number is the starting paramter value
%  the second number is the lower range to search
%  the third number is the upper range to search
guess0_(1,:)   =[0.0011 1e-7 0.5];   % Z - the viscosity scaling factor of network (B), see JASA paper Eq. (9) (Zhang et al., 2006)
guess0_(2,:)   =[-0.99792 -1 -0.01];   % c - the stretch exponent of network (B), set to -1, see JASA paper Eq. (9) (Zhang et al., 2006)
guess0_(3,:)   =[1.2118 0.1 50];      % m_B - the stress exponent of network (B), see JASA paper Eq. (9) (Zhang et al., 2006)
guess0_(4,:)   =[29.165 0.01 50];         % mu_A - the initial shear modulus of equilibrium network (A), see JASA paper Eq. (6) (Zhang et al., 2006)
guess0_(5,:)   =[193.6132 1 550];     % mu_B - the initial shear modulus of short-term network (B), see JASA paper Eq. (12) (Zhang et al., 2006)
guess0_(6,:)   =[19.6836 10 40];        %a - alpha; dimensionless power describing the nonlinearity, see JASA paper Eq. (6) (Zhang et al., 2006)


fig=100;             % figure number to start plots at

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% fit data and find the values
%S=load('anilogden.txt');
%load s9sinPCAv5.0.wdq.mat;
%eps_ = interp(eps_,3);
%sig_ = interp(sig_,3);
%eps_ = S(:,1);
%sig_ = S(:,2);
eps_ = [xlsread('left_cover_4v_66-130cycle.xls','sheet2','BG9:BG1008')]/100;
% eps2 = xlsread('left_cover_4v_66-130cycle.xls','sheet2','u9:u1008');
sig_ = xlsread('left_cover_4v_66-130cycle.xls','sheet2','BH9:BH1008');
% sig2 = xlsread('left_cover_4v_66-130cycle.xls','sheet2','v9:v1008');
% sig_ = [sig1;sig2];
% eps_ = [eps1;eps2]/100;
%load s9sinPCAv5.0.wdq.mat
tau_ = 1/1000;   %sampling rate

%plot of stress vs strain from data
figure(fig), fig=fig+1;
arry=1:length(eps_);
%arry=arry(round(1):end);
plot(100*eps_(arry),sig_(arry),'k.')
xlabel('Strain (%)')
ylabel('Stress (kPa)')
legend('Data',2)
title('Stress Strain Data')

%%%%% fit the data
lamda_A=1+eps_+0.0001;
[tmp]=getbananafit_free(guess0_,sig_,lamda_A,tau_,num_itt_,fig);
guessKelFull_=tmp;  % fit parameter values

%%%%% get the guess from the fit
dlamda_A=0.*lamda_A;
dlamda_A(2:end)=diff(lamda_A)/tau_;	% first point is ignored
dlamda_A(1)=dlamda_A(2);
sigf=model_bananafunc([guessKelFull_],tau_,lamda_A,dlamda_A); %kPa    

%%%%% Plot of stress vs strain from fit
titles=['data';'fit '];                                                       
set(0,'DefaultAxesLineStyleOrder',{'.','-'})
set(0,'DefaultAxesColorOrder',[0.,0.,0.])
figure(fig), fig=fig+1;
plot(100*eps_(arry),sig_(arry),100*eps_(arry),sigf(arry))
legend(titles,'Location','NorthWest')

% save the calculated fit data into a file.
gf=gfit2(sigf,sig_');
fid = fopen('good_fiteps_30.txt','wt');
fprintf(fid,'%12.8f \n',gf');
fclose(fid);

keyboard


%% solve for lamda_B and sigT
function sigT = model_bananafunc(par,tau,lamda_A,dlamda_A)
% Function to solve for lamda_B and obtain total stress data
% par - seed parameters
% tau - sampling rate
% lamda_A - strech of network A
% dlamda_A - derivative of strech of network A
%
% returns total stress in network A and network B
%%%%%%%%%%%%%%%
%
%   Anil Palaparthi 2010-02-08
%

lamda_B=1;  % Initial value for lamda_B

%Store the parameters in temporary variables
Z = par(1,1);c = par(1,2); m_B = par(1,3); mu_A = par(1,4); mu_B = par(1,5);
a = par(1,6);

% Initial value for derivative of lamda_B
dlamda_B = [dlamda_A(1)*(lamda_B/lamda_A(1))]-...
           (2/3)*lamda_B*Z*sign(lamda_B-1+1e-15)*...
           [[sqrt((1/3)*[(lamda_A(1)/lamda_B)^2+(2*lamda_B/lamda_A(1))])-1+1e-4]^c]*...
           [abs((2*mu_B/a)*[(lamda_B^a)-(lamda_B^(-0.5*a))])]^m_B;
 
ns = length(lamda_A);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% initializations for RK4for.m DO NOT CHANGE ANYTHING %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
phi = lamda_B;  % RK routine internal variable
yo = phi;       % RK routine internal variable
N = 1;          % number of equations solved by RK4
tmp = 1;        % temporary counting array
c1 = tmp+1;     % indecies for regaining above variables
c3 = tmp+1+2*N; c4 = tmp+1+3*N;  %_from RK4 output 
time = 0;       % time set to zero,
sigT = zeros(1,ns);  %Initialize total stress
sigT(1) = (2*mu_A/a)*[(lamda_A(1))^(a-1)-(lamda_A(1))^(-0.5*a-1)]+...
    (2*mu_B/a)*[(lamda_B)^(a-1)-(lamda_B)^(-0.5*a-1)];

for n=2:ns
    
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %% Runge-Kutta 4 routine...
   %%
   
   Istat = 1;
   m = 0;
   while(Istat~=0),
       m = m+1;
       [temp] = RKfor(N,lamda_B, dlamda_B, time, tau, phi, yo, m);
       %% send [(N,F,dF,time,tau,phi,yo,m)
       % ...N = number of variables to solve, 
       % ...F = variable to solve
       % ...dF= its derivative
       % ...time= current time in iteration
       % ...tau = time step,
       % ...internal parameter for recall
       % ...internal parameter for recall].
       % ...Returns updated... [status F dF phi yo time]
       %% regain variables 
       Istat = temp(1);     %% condition of loop
       lamda_B = temp(c1);  %% update the RKA variables
       phi = temp(c3);      %% internal variable for recall
       yo = temp(c4);       %% internal variable for recall
       time = temp(end);    %% time update
       
       % derivative of lamda_B: equation 11 from (Zhang et al., 2006)
       dlamda_B(1) = [dlamda_A(n)*(lamda_B(1)/lamda_A(n-1))]-...
           [(2/3)*lamda_B(1)*Z*sign(lamda_B(1)-1+1e-15)*...
           [[sqrt((1/3)*[(lamda_A(n-1)/lamda_B(1))^2+(2*lamda_B(1)/lamda_A(n-1))])-1+1e-4]^c]*...
           [abs((2*mu_B/a)*[(lamda_B(1)^a)-(lamda_B(1)^(-0.5*a))])]^m_B];
 
   end
   %%
   %%  end runge-kutta 
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
      
   % Total Stress for network A and network B
   sigT(n) = (2*mu_A/a)*[(lamda_A(n))^(a-1)-(lamda_A(n))^(-0.5*a-1)]+...
       (2*mu_B/a)*[(lamda_B)^(a-1)-(lamda_B)^(-0.5*a-1)]; 
end
return
     



function [x,fval,exitflag,output]=fminsearchbnd(fun,x0,LB,UB,options,varargin)
% fminsearchbnd: fminsearch, but with bound constraints by transformation
% usage: fminsearchbnd(fun,x0,LB,UB,options,p1,p2,...)
% 
% arguments:
%  LB - lower bound vector or array, must be the same size as x0
%
%       If no lower bounds exist for one of the variables, then
%       supply -inf for that variable.
%
%       If no lower bounds at all, then LB may be left empty.
%
%  UB - upper bound vector or array, must be the same size as x0
%
%       If no upper bounds exist for one of the variables, then
%       supply +inf for that variable.
%
%       If no upper bounds at all, then UB may be left empty.
%
%  See fminsearch for all other arguments and options.
%  Note that TolX will apply to the transformed variables. All other
%  fminsearch parameters are unaffected.
%
% Notes:
%
%  Variables which are constrained by both a lower and an upper
%  bound will use a sin transformation. Those constrained by
%  only a lower or an upper bound will use a quadratic
%  transformation, and unconstrained variables will be left alone.
%
%  Variables may be fixed by setting their respective bounds equal.
%  In this case, the problem will be reduced in size for fminsearch.
%
%  The bounds are inclusive inequalities, which admit the
%  boundary values themselves, but will not permit ANY function
%  evaluations outside the bounds.
%
%
% Example usage:
% rosen = @(x) (1-x(1)).^2 + 105*(x(2)-x(1).^2).^2;
%
% fminsearch(rosen,[3 3])     % unconstrained
% ans =
%    1.0000    1.0000
%
% fminsearchbnd(rosen,[3 3],[2 2],[])     % constrained
% ans =
%    2.0000    4.0000
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    	Title:  	fminsearchbnd
%   	Author: 	John D'Errico
%    	File Id:  	8277  	Average rating:  	5.0
%   	Size: 	6 KB 	# of reviews: 	6
%   	Submitted: 	2005-08-11 	Downloads: 	963
%   	Subscribers: 	2 	 
%   	Keywords: 	Bound constraints
%   	Other Requirements: 	This code should work on any release of matlab which allows the use of sub-functions.
%     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  edited by eric hunter @ncvs.org, 20060223
%

t0=clock;


% size checks
xsize = size(x0);
x0 = x0(:);
n=length(x0);

if (nargin<3) || isempty(LB)
  LB = repmat(-inf,n,1);
else
  LB = LB(:);
end
if (nargin<4) || isempty(UB)
  UB = repmat(inf,n,1);
else
  UB = UB(:);
end

if (n~=length(LB)) | (n~=length(UB))
  error 'x0 is incompatible in size with either LB or UB.'
end

% set default options if necessary
if (nargin<5)|isempty(options)
  options = optimset('fminsearch');
end

% stuff into a struct to pass around
params.args = varargin;
params.LB = LB;
params.UB = UB;
params.fun = fun;
params.n = n;

% 0 --> unconstrained variable
% 1 --> lower bound only
% 2 --> upper bound only
% 3 --> dual finite bounds
% 4 --> fixed variable
params.BoundClass = zeros(n,1);
for i=1:n
  k = isfinite(LB(i)) + 2*isfinite(UB(i));
  params.BoundClass(i) = k;
  if (k==3) & (LB(i)==UB(i))
    params.BoundClass(i) = 4;
  end
end

% transform starting values into their unconstrained
% surrogates. Check for infeasible starting guesses.
x0u = x0;
k=1;
for i = 1:n
  switch params.BoundClass(i)
    case 1
      % lower bound only
      if x0(i)<=LB(i)
        % infeasible starting value. Use bound.
        x0u(k) = 0;
      else
        x0u(k) = sqrt(x0(i) - LB(i));
      end
      
      % increment k
      k=k+1;
    case 2
      % upper bound only
      if x0(i)>=UB(i)
        % infeasible starting value. use bound.
        x0u(k) = 0;
      else
        x0u(k) = sqrt(UB(i) - x0(i));
      end
      
      % increment k
      k=k+1;
    case 3
      % lower and upper bounds
      if x0(i)<=LB(i)
        % infeasible starting value
        x0u(k) = -pi/2;
      elseif x0(i)>=UB(i)
        % infeasible starting value
        x0u(k) = pi/2;
      else
        x0u(k) = 2*(x0(i) - LB(i))/(UB(i)-LB(i)) - 1;
        % shift by 2*pi to avoid problems at zero in fminsearch
        % otherwise, the initial simplex is vanishingly small
        x0u(k) = 2*pi+asin(max(-1,min(1,x0u(i))));
      end
      
      % increment k
      k=k+1;
    case 0
      % unconstrained variable. x0u(i) is set.
      x0u(k) = x0(i);
      
      % increment k
      k=k+1;
    case 4
      % fixed variable. drop it before fminsearch sees it.
      % k is not incremented for this variable.
  end
  
end
% if any of the unknowns were fixed, then we need to shorten
% x0u now.
if k<=n
  x0u(k:n) = [];
end

% were all the variables fixed?
if isempty(x0u)
  % All variables were fixed. quit immediately, setting the
  % appropriate parameters, then return.
  
  % undo the variable transformations into the original space
  x = xtransform(x0u,params);
  
  % final reshape
  x = reshape(x,xsize);
  
  % stuff fval with the final value
  fval = feval(params.fun,x,params.args{:});
  
  % fminsearchbnd was not called
  exitflag = 0;
  
  output.iterations = 0;
  output.funcount = 1;
  output.algorithm = 'myfminsearch';
  output.message = 'All variables were held fixed by the applied bounds';
  
  % return with no call at all to fminsearch
  return
end

% now we can call fminsearch, but with our own
% intra-objective function.
[xu,fval,exitflag,output] = myfminsearch(@intrafun,x0u,options,params);

% undo the variable transformations into the original space
x = xtransform(xu,params);

% final reshape
x = reshape(x,xsize);

disp(['Elapse time: ' num2str(etime(clock,t0)) ' sec.']);
return

% ======================================
% ========= begin subfunctions =========
% ======================================
function fval = intrafun(x,params)
% transform variables, then call original function

% transform
xtrans = xtransform(x,params);

% and call fun
fval = feval(params.fun,xtrans,params.args{:});
return


% ======================================
function xtrans = xtransform(x,params)
% converts unconstrained variables into their original domains

xtrans = zeros(1,params.n);
% k allows soem variables to be fixed, thus dropped from the
% optimization.
k=1;
for i = 1:params.n
  switch params.BoundClass(i)
    case 1
      % lower bound only
      xtrans(i) = params.LB(i) + x(k).^2;
      
      k=k+1;
    case 2
      % upper bound only
      xtrans(i) = params.UB(i) - x(k).^2;
      
      k=k+1;
    case 3
      % lower and upper bounds
      xtrans(i) = (sin(x(k))+1)/2;
      xtrans(i) = xtrans(i)*(params.UB(i) - params.LB(i)) + params.LB(i);
      % just in case of any floating point problems
      xtrans(i) = max(params.LB(i),min(params.UB(i),xtrans(i)));
      
      k=k+1;
    case 4
      % fixed variable, bounds are equal, set it at either bound
      xtrans(i) = params.LB(i);
    case 0
      % unconstrained variable.
      xtrans(i) = x(k);
      
      k=k+1;
  end
end
return


function [guess1_]=getbananafit_free(guess_,sig_,lamda_A,tau_,num_itt_,fig)
%getbananafit	gets the fit of the work cycle.
%   guess_(1)   =0.99;   % guess 
%   %   sig_    % stress data to be fit to
%   eps_    % strain data corresponding to stress
%   tau_    % time between samples
%   num_itt_    % upper limit on the number of optimizing loops
%
%   Returns the fit guess_ parameters
%
%%%%%%%
%
%	Eric Hunter; May 2002
%   Anil Palaparthi: 2010-02-08
%

guess0_=guess_(:,1)';

dlamda_A=0.*lamda_A;
dlamda_A(2:end)=diff(lamda_A)/tau_;	% first point is ignored
dlamda_A(1)=dlamda_A(2);
eps_ = lamda_A-1-0.0001;
arry=1:length(eps_);
%arry=arry(round(1):end);
sigf_= model_bananafunc([guess_(:,1)'],tau_,lamda_A,dlamda_A);
figure(fig)
plot(100*eps_(arry),sig_(arry),'k.'...
    ,100*eps_(arry),sigf_(arry),'ko')
xlabel('Strain (%)')
ylabel('Stress (kPa)')
legend('Data','Guess',2)
hold on
plot(100*eps_(arry),sig_(arry),'k'...
    ,100*eps_(arry),sigf_(arry),'k')
hold off

options.MaxFunEvals=(num_itt_);
options.MaxIter=(length(guess_)*num_itt_);
options.TolX=1e-4;
options.TolFun=1e-4;

[guess1_ tmp1 tmp2 output]	= fminsearchbnd('fitfun_banana_eps1',...
   [guess_(:,1)'],[guess_(:,2)'],[guess_(:,3)'],options,sig_',tau_,lamda_A,dlamda_A);

disp(' ')
disp(' ------------------------------------------------------------- ')
disp(' ')
disp(['#fun     ---    ' num2str(output.funcCount)])
disp(['#itt     ---    ' num2str(output.iterations)])
disp(['Var      Start      Fit'])
disp(['Z    ' num2str(guess0_(1)) '    ' num2str(guess1_(1))])
disp(['c    ' num2str(guess0_(2)) '    ' num2str(guess1_(2))])
disp(['m    ' num2str(guess0_(3)) '    ' num2str(guess1_(3))])
disp(['mu_A    ' num2str(guess0_(4)) '    ' num2str(guess1_(4))])
disp(['mu_B       ' num2str(guess0_(5)) '    ' num2str(guess1_(5))])
disp(['a      ' num2str(guess0_(6)) '    ' num2str(guess1_(6))])
disp(' ')
disp(' ------------------------------------------------------------- ')
disp(' ')
return


%----------------------------------------------------------



function out = RKfor( N,Y,F,X,H,PHI,Y0,m) 
%RKfor
%
%	Runge-Kutte 4th order routine based on a fortran 
%		version used by Ingo Titze.  
%
%	Eric Hunter did the conversion
%
J=1:N;
switch m
case 1,
   istat=1;
case 2,
   istat=1;
   Y0(J)=Y(J);
   PHI(J)=F(J);
   Y(J)=Y0(J)+0.5*H*F(J);
   X=X+0.5*H;
case 3,
   istat=1;
   PHI(J)=PHI(J)+2.*F(J);
   Y(J)=Y0(J)+0.5*H*F(J);
case 4,
   istat=1;
   PHI(J)=PHI(J)+2.*F(J);
   Y(J)=Y0(J)+H*F(J);
   X=X+0.5*H;
otherwise,
   istat=0;
   Y(J)=Y0(J)+(PHI(J)+F(J))*H/6;           
end
out=[istat,Y,F,PHI,Y0,X];
return
%----------------------------------------------------------


function s_sq_error=fitfun_banana_eps1(guess,sigp,tau,lamda_A,dlamda_A)
%FITFUN_BANANA	passive kelvin model of tissue compare.
%   send a guess (passive kelvin parameters),
%   the strain (epsy) and the 
%   stress to compare it to stress (sigp), 
%
%%%%%%%
%
%	Eric Hunter; May 2002
%

%sigp2=model_bananafun([guess(1),guess(2),-0.5,guess(3),guess(4)],1,epsy); 
% sigp2=model_bananafun([guess(1),guess(2),guess(3),guess(4),guess(5),guess
% (6),guess(7)],tau,epsy,depsy);
sigp2=model_bananafunc([guess],tau,lamda_A,dlamda_A);
%IA,LCA,PCAo,PCAv
% length(sigp)
% length(sigp2)
% length(epsy)

Error_Vector=(sigp - sigp2);  
s_sq_error=sum(Error_Vector(floor(1):floor(end)).^2);  	
return
%----------------------------------------------------------

function [gf] = gfit2(t,y)
% GFIT2 Computes goodness of fit
%
% USAGE:
%       [gf] = gfit2(t,y) 
%
% INPUT:
%           t:  vector of target values for regression model
%           y:  vector of output from regression model.
%
%               '1' - mean squarred error (mse)  
%               '2' - normalised mean squarred error (nmse)
%               '3' - root mean squarred error (rmse)
%               '4' - normalised root mean squarred error (nrmse)
%               '5' - mean absolute error (mae)
%               '6' - mean  absolute relative error  (mare)
%               '7' - coefficient of correlation (r)
%               '8' - coefficient of determination (d)
%               '9' - coefficient of efficiency (e)         
%              
% OUTPUT:
%       gf:  goodness of fit values between model output and target 
%
% EXAMPLES   
%      gf = git2(t,y);  for root mean squarred error    
%
% Eric Hunter, 20060220, NCVS.org
% modified from a script submitted to MATLAB.com by...
%
% Copyright 2004-2005 by Durga Lal Shrestha.
%

e= t - y;       % residual

%'1' - mean squarred error (mse)  
gf(1) = mean(e.^2);        % 0 - perfect match between output and target

%'2' - normalised mean squarred error (nmse)
gf(2) = gf(1)/var(t); % 0 - perfect match 

%'3' - root mean squarred error (rmse)
gf(3) = sqrt(gf(1));  % 0 - perfect match     

%'4' - normalised root mean squarred error (nrmse)
gf(4) = sqrt(gf(1)/var(t)); % 0 - perfect match

%'5' - mean absolute error (mae)
gf(5) = mean(abs(e));       % 0 - perfect match

%'6' - mean  absolute relative error  (mare)
gf(6) = mean((abs(e./t)));  % 0 - perfect match

%'7' - coefficient of correlation (r)
cf = corrcoef(t,y);      % 1 - perfect match
gf(7) = cf(1,2);     

%'8' - coefficient of determination (d)
cf = corrcoef(t,y);      
tmp = cf(1,2);
gf(8) = tmp^2;               % 1 - perfect match

%'9' - coefficient of efficiency (e)  
gf(9) = 1 - sum(e.^2)/sum((t - mean(t)).^2); % 1 - perfect match
   

%'10'- r-square: statistical measure indicating how
%      successful the fit is in explaining the data.  in other
%      words, r-square=correlation between data and fit.
sse=sum((t-y).^2);
sst=sum((t-mean(t)).^2);
rsqrt=1-sse/sst;
gf(10)=rsqrt;
return
